//ENode.cpp
#include "ENode.h"
#include "../xmlnode/XmlNode.h"
#include "XmlMesh.h"
#include "../GameTime.h"

ENode::ENode( LPDIRECT3DDEVICE9 pDevice )
: m_pParent( NULL ) 
, m_child( NULL )
{
	m_pDevice = pDevice;
	D3DXMatrixIdentity( &m_matWorld );
	D3DXMatrixIdentity( &m_matLocal );
	m_pAniInfo = NULL;
	m_aniArray = NULL;
	m_nAniSize = 0;
}
//-----------------------------------------------------------------------------
ENode::~ENode()
{
	//ڽ 
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		delete (*iter);
	}
	m_child.clear();

	if( m_pAniInfo )
	{
		delete m_pAniInfo;
		m_pAniInfo = NULL;
	}

	if( m_aniArray )
	{
		delete [] m_aniArray;
		m_aniArray = NULL;
		m_nAniSize = 0;
	}
}
//-----------------------------------------------------------------------------
const char* ENode::GetClassName()
{
	return "ENode";
}
//-----------------------------------------------------------------------------
std::string* ENode::GetName()
{
	return &m_name;
}
//-----------------------------------------------------------------------------
void ENode::SetName( const char* szName )
{
	m_name = szName;
}
//-----------------------------------------------------------------------------
ENode* ENode::AttachChild( ENode* pChild )
{
	pChild->m_pParent = this;
	m_child.push_back( pChild );
	return pChild;
}
//-----------------------------------------------------------------------------
void ENode::DetachChild( ENode* pChild )
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		if( pChild == *iter )
		{
			delete (*iter);
			m_child.erase( iter );
			break;
		}
	}
}
//-----------------------------------------------------------------------------
ENode* ENode::GetParent()
{
	return m_pParent;
}
//-----------------------------------------------------------------------------
ENode* ENode::GetRoot()
{
	return m_pParent ? m_pParent->GetRoot() : this;
}
//-----------------------------------------------------------------------------
void ENode::Draw()
{
	OnCalcWorldMatrix();
	OnDraw();
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		ENode* pChild = (ENode*)*iter;
		pChild->Draw();
	}	
}
//-----------------------------------------------------------------------------
void ENode::OnCalcWorldMatrix()
{
	if( m_pParent == NULL )
	{
		m_matWorld = m_matLocal;
	}
	else
	{
		if( this->m_nAniSize > 0 )
		{
			int nFrame = GameTime::GetFrame();	
			m_matWorld = m_aniArray[nFrame] * m_pParent->m_matWorld;
		}
		else
			m_matWorld = m_matLocal * m_pParent->m_matWorld;
	}
}
//-----------------------------------------------------------------------------
void ENode::SetMatrix( const D3DXMATRIXA16& mat )
{
	m_matLocal = mat;
}
//-----------------------------------------------------------------------------
void ENode::CreateAnimation( XmlMesh* pXmlMesh )
{
	int nSize = m_nAniSize = pXmlMesh->m_nAniSize;
	m_aniArray = new D3DXMATRIXA16[ nSize ];

	for( int i = 0; i < nSize; ++i )
	{
		m_aniArray[i] = pXmlMesh->m_aniArray[i];
	}
}
//-----------------------------------------------------------------------------
void ENode::CreateAnimationHeader( XmlMesh* pXmlMesh )
{
	assert( m_pAniInfo == NULL );
	AnimationInfo* pAniInfo = pXmlMesh->m_pAniInfo;
	if( pAniInfo == NULL )
		return;
	m_pAniInfo = new AnimationInfo;
	m_pAniInfo->beginFrame = pAniInfo->beginFrame;
	m_pAniInfo->endFrame = pAniInfo->endFrame;
	m_pAniInfo->fps = pAniInfo->fps;
	m_pAniInfo->tickPerFrame = pAniInfo->tickPerFrame;

	GameTime::SetFPS( m_pAniInfo->fps );
	GameTime::SetEndFrame( m_pAniInfo->endFrame );
}